@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 #
-# Copyright (c), 2018-2019, SISSA (International School for Advanced Studies).
+# Copyright (c), 2018-2020, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -8,10 +7,10 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-__version__ = '1.4.0'
+__version__ = '1.4.1'
 __author__ = "Davide Brunato"
 __contact__ = "brunato@sissa.it"
-__copyright__ = "Copyright 2018-2019, SISSA"
+__copyright__ = "Copyright 2018-2020, SISSA"
 __license__ = "MIT"
 __status__ = "Production/Stable"
 
@@ -29,3 +28,11 @@ from .xpath1_parser import XPath1Parser
 from .xpath2_constructors import XPath2Parser
 from .xpath_selectors import select, iter_select, Selector
 from .schema_proxy import AbstractSchemaProxy
+
+__all__ = ['ElementPathError', 'MissingContextError', 'ElementPathSyntaxError',
+           'ElementPathNameError', 'ElementPathKeyError', 'ElementPathTypeError',
+           'ElementPathLocaleError', 'ElementPathValueError', 'datatypes', 'Token',
+           'Parser', 'XPathContext', 'XPathSchemaContext', 'AttributeNode', 'TextNode',
+           'TypedAttribute', 'TypedElement', 'NamespaceNode', 'XPathToken',
+           'XPath1Parser', 'XPath2Parser', 'select', 'iter_select', 'Selector',
+           'AbstractSchemaProxy']
