@@ -1,5 +1,5 @@
 #
-# Copyright (c), 2018-2019, SISSA (International School for Advanced Studies).
+# Copyright (c), 2018-2020, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -23,8 +23,10 @@ from .xpath_nodes import NamespaceNode, TypedAttribute, TypedElement, is_etree_e
     is_comment_node, is_processing_instruction_node, node_name
 
 
-XML_NAME_CHARACTER = (u"A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF"
-                      u"\u200C\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD")
+XML_NAME_CHARACTER = (
+    u"A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF"
+    u"\u200C\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD"
+)
 XML_NCNAME_PATTERN = u"[{0}][-.0-9\u00B7\u0300-\u036F\u203F-\u2040{0}]*".format(XML_NAME_CHARACTER)
 
 
@@ -375,7 +377,8 @@ def nud(self):
     namespace = self.parser.next_token.value + self.parser.raw_advance('}')
     self.parser.advance()
     self.parser.next_token.bind_namespace(namespace)
-    self[:] = self.parser.symbol_table['(string)'](self.parser, namespace), self.parser.expression(90)
+    self[:] = self.parser.symbol_table['(string)'](self.parser, namespace), \
+        self.parser.expression(90)
     return self
 
 
@@ -452,10 +455,11 @@ def select(self, context=None):
 def select(self, context=None):
     if context is None:
         self.missing_context()
-    elif context.item is not None:
-        yield context.item
-    elif is_document_node(context.root):
-        yield context.root
+    for item in context.iter_self():
+        if item is not None:
+            yield item
+        elif is_document_node(context.root):
+            yield context.root
 
 
 @method(nullary('..'))
