@@ -1,5 +1,5 @@
 #
-# Copyright (c), 2018-2019, SISSA (International School for Advanced Studies).
+# Copyright (c), 2018-2020, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -165,7 +165,9 @@ class XPathToken(Token):
                 if k == 0:
                     item = result
                 elif not self.parser.compatibility_mode:
-                    self.wrong_context_type("a sequence of more than one item is not allowed as argument")
+                    self.wrong_context_type(
+                        "a sequence of more than one item is not allowed as argument"
+                    )
                 else:
                     break
             else:
@@ -213,7 +215,8 @@ class XPathToken(Token):
         for item in self.select(context):
             value = self.data_value(item)
             if value is None:
-                raise self.error('FOTY0012', "argument node {!r} does not have a typed value".format(item))
+                msg = "argument node {!r} does not have a typed value"
+                raise self.error('FOTY0012', msg.format(item))
             else:
                 yield value
 
@@ -518,19 +521,19 @@ class XPathToken(Token):
 
     @contextlib.contextmanager
     def use_locale(self, collation):
-        """A context manager for setting a specific collation for a code block."""
-        loc = locale.getlocale()
+        """A context manager for use a locale setting for string comparison in a code block."""
+        loc = locale.getlocale(locale.LC_COLLATE)
         if collation == UNICODE_CODEPOINT_COLLATION:
             collation = 'en_US.UTF-8'
 
         try:
-            locale.setlocale(locale.LC_ALL, collation)
+            locale.setlocale(locale.LC_COLLATE, collation)
         except locale.Error:
             raise self.error('FOCH0002', 'Unsupported collation %r' % collation)
         else:
             yield
         finally:
-            locale.setlocale(locale.LC_ALL, loc)
+            locale.setlocale(locale.LC_COLLATE, loc)
 
     ###
     # XPath data accessors base functions
@@ -568,7 +571,8 @@ class XPathToken(Token):
                             "more items not starting with an XPath node.",
                 )
         elif isinstance(obj, tuple) or is_element_node(obj):
-            raise self.error('FORG0006', "Effective boolean value is not defined for {}.".format(obj))
+            msg = "Effective boolean value is not defined for {}."
+            raise self.error('FORG0006', msg.format(obj))
         return bool(obj)
 
     def string_value(self, obj):
